@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Google Inc.
+ * Copyright 2014 Google LLC
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,73 +15,58 @@
  */
 package com.google.auto.value.processor;
 
-import static com.google.common.truth.Truth.assertAbout;
-import static com.google.common.truth.Truth.assertThat;
-import static com.google.common.truth.Truth.assert_;
-import static com.google.testing.compile.JavaSourcesSubjectFactory.javaSources;
-import static org.junit.Assert.assertTrue;
+import static com.google.common.truth.Truth8.assertThat;
+import static com.google.testing.compile.CompilationSubject.assertThat;
+import static com.google.testing.compile.Compiler.javac;
 
-import com.google.common.collect.ImmutableList;
+import com.google.testing.compile.Compilation;
 import com.google.testing.compile.JavaFileObjects;
-import java.io.File;
-import java.io.IOException;
-import java.io.Writer;
-import java.util.List;
-import javax.tools.Diagnostic;
-import javax.tools.DiagnosticCollector;
-import javax.tools.JavaCompiler;
-import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
-import javax.tools.ToolProvider;
-import org.junit.Rule;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/**
- * @author emcmanus@google.com (Éamonn McManus)
- */
+/** @author emcmanus@google.com (Éamonn McManus) */
 @RunWith(JUnit4.class)
 public class AutoAnnotationCompilationTest {
 
-  @Rule public final GeneratedImportRule generatedImportRule = new GeneratedImportRule();
-
   @Test
   public void testSimple() {
-    JavaFileObject myAnnotationJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.annotations.MyAnnotation",
-        "package com.example.annotations;",
-        "",
-        "import com.example.enums.MyEnum;",
-        "",
-        "public @interface MyAnnotation {",
-        "  MyEnum value();",
-        "  int defaultedValue() default 23;",
-        "}"
-    );
+    JavaFileObject myAnnotationJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.annotations.MyAnnotation",
+            "package com.example.annotations;",
+            "",
+            "import com.example.enums.MyEnum;",
+            "",
+            "public @interface MyAnnotation {",
+            "  MyEnum value();",
+            "  int defaultedValue() default 23;",
+            "}");
     int invariableHash = ("defaultedValue".hashCode() * 127) ^ 23;
-    JavaFileObject myEnumJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.enums.MyEnum",
-        "package com.example.enums;",
-        "",
-        "public enum MyEnum {",
-        "  ONE",
-        "}"
-    );
-    JavaFileObject annotationFactoryJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.factories.AnnotationFactory",
-        "package com.example.factories;",
-        "",
-        "import com.google.auto.value.AutoAnnotation;",
-        "import com.example.annotations.MyAnnotation;",
-        "import com.example.enums.MyEnum;",
-        "",
-        "public class AnnotationFactory {",
-        "  @AutoAnnotation",
-        "  public static MyAnnotation newMyAnnotation(MyEnum value) {",
-        "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value);",
-        "  }",
-        "}");
+    JavaFileObject myEnumJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.enums.MyEnum",
+            "package com.example.enums;",
+            "",
+            "public enum MyEnum {",
+            "  ONE",
+            "}");
+    JavaFileObject annotationFactoryJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.factories.AnnotationFactory",
+            "package com.example.factories;",
+            "",
+            "import com.google.auto.value.AutoAnnotation;",
+            "import com.example.annotations.MyAnnotation;",
+            "import com.example.enums.MyEnum;",
+            "",
+            "public class AnnotationFactory {",
+            "  @AutoAnnotation",
+            "  public static MyAnnotation newMyAnnotation(MyEnum value) {",
+            "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value);",
+            "  }",
+            "}");
     JavaFileObject expectedOutput =
         JavaFileObjects.forSourceLines(
             "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation",
@@ -89,10 +74,13 @@ public class AutoAnnotationCompilationTest {
             "",
             "import com.example.annotations.MyAnnotation;",
             "import com.example.enums.MyEnum;",
-            generatedImportRule.importGeneratedAnnotationType(),
+            "import java.io.Serializable;",
+            GeneratedImport.importGeneratedAnnotationType(),
             "",
             "@Generated(\"" + AutoAnnotationProcessor.class.getName() + "\")",
-            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation {",
+            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation",
+            "     implements MyAnnotation, Serializable {",
+            "  private static final long serialVersionUID = -7473814294717163169L;",
             "  private final MyEnum value;",
             "  private static final int defaultedValue = 23;",
             "",
@@ -127,7 +115,7 @@ public class AutoAnnotationCompilationTest {
             "    }",
             "    if (o instanceof MyAnnotation) {",
             "      MyAnnotation that = (MyAnnotation) o;",
-            "      return (value.equals(that.value()))",
+            "      return value.equals(that.value())",
             "          && (defaultedValue == that.defaultedValue());",
             "    }",
             "    return false;",
@@ -136,40 +124,49 @@ public class AutoAnnotationCompilationTest {
             "  @Override public int hashCode() {",
             "    return ",
             "        " + invariableHash,
-            "        + (" + 127 * "value".hashCode() + " ^ (value.hashCode()))",
+            "        + (" + 127 * "value".hashCode() + " ^ value.hashCode())",
             "    ;",
             "  }",
             "}");
-    assert_().about(javaSources())
-        .that(ImmutableList.of(annotationFactoryJavaFile, myAnnotationJavaFile, myEnumJavaFile))
-        .processedWith(new AutoAnnotationProcessor())
-        .compilesWithoutError()
-        .and().generatesSources(expectedOutput);
+    Compilation compilation =
+        javac()
+            .withProcessors(new AutoAnnotationProcessor())
+            .withOptions("-A" + Nullables.NULLABLE_OPTION + "=")
+            .compile(annotationFactoryJavaFile, myAnnotationJavaFile, myEnumJavaFile);
+    assertThat(compilation).succeededWithoutWarnings();
+    assertThat(compilation)
+        .generatedSourceFile(
+            "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation")
+        .hasSourceEquivalentTo(expectedOutput);
   }
 
   @Test
   public void testEmptyPackage() {
-    JavaFileObject myAnnotationJavaFile = JavaFileObjects.forSourceLines(
-        "MyAnnotation",
-        "public @interface MyAnnotation {}"
-    );
-    JavaFileObject annotationFactoryJavaFile = JavaFileObjects.forSourceLines(
-        "AnnotationFactory",
-        "import com.google.auto.value.AutoAnnotation;",
-        "",
-        "public class AnnotationFactory {",
-        "  @AutoAnnotation",
-        "  public static MyAnnotation newMyAnnotation() {",
-        "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation();",
-        "  }",
-        "}");
+    JavaFileObject myAnnotationJavaFile =
+        JavaFileObjects.forSourceLines(
+            "MyAnnotation", //
+            "public @interface MyAnnotation {}");
+    JavaFileObject annotationFactoryJavaFile =
+        JavaFileObjects.forSourceLines(
+            "AnnotationFactory",
+            "import com.google.auto.value.AutoAnnotation;",
+            "",
+            "public class AnnotationFactory {",
+            "  @AutoAnnotation",
+            "  public static MyAnnotation newMyAnnotation() {",
+            "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation();",
+            "  }",
+            "}");
     JavaFileObject expectedOutput =
         JavaFileObjects.forSourceLines(
             "AutoAnnotation_AnnotationFactory_newMyAnnotation",
-            generatedImportRule.importGeneratedAnnotationType(),
+            "import java.io.Serializable;",
+            GeneratedImport.importGeneratedAnnotationType(),
             "",
             "@Generated(\"" + AutoAnnotationProcessor.class.getName() + "\")",
-            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation {",
+            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation",
+            "    implements MyAnnotation, Serializable {",
+            "  private static final long serialVersionUID = 0L;",
             "  AutoAnnotation_AnnotationFactory_newMyAnnotation() {",
             "  }",
             "",
@@ -196,56 +193,64 @@ public class AutoAnnotationCompilationTest {
             "    return 0;",
             "  }",
             "}");
-    assertAbout(javaSources())
-        .that(ImmutableList.of(annotationFactoryJavaFile, myAnnotationJavaFile))
-        .processedWith(new AutoAnnotationProcessor())
-        .compilesWithoutError()
-        .and().generatesSources(expectedOutput);
+    Compilation compilation =
+        javac()
+            .withProcessors(new AutoAnnotationProcessor())
+            .withOptions("-A" + Nullables.NULLABLE_OPTION + "=")
+            .compile(annotationFactoryJavaFile, myAnnotationJavaFile);
+    assertThat(compilation).succeededWithoutWarnings();
+    assertThat(compilation)
+        .generatedSourceFile("AutoAnnotation_AnnotationFactory_newMyAnnotation")
+        .hasSourceEquivalentTo(expectedOutput);
   }
 
   @Test
   public void testGwtSimple() {
-    JavaFileObject myAnnotationJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.annotations.MyAnnotation",
-        "package com.example.annotations;",
-        "",
-        "import com.google.common.annotations.GwtCompatible;",
-        "",
-        "@GwtCompatible",
-        "public @interface MyAnnotation {",
-        "  int[] value();",
-        "}"
-    );
-    JavaFileObject gwtCompatibleJavaFile = JavaFileObjects.forSourceLines(
-        "com.google.common.annotations.GwtCompatible",
-        "package com.google.common.annotations;",
-        "",
-        "public @interface GwtCompatible {}"
-    );
-    JavaFileObject annotationFactoryJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.factories.AnnotationFactory",
-        "package com.example.factories;",
-        "",
-        "import com.google.auto.value.AutoAnnotation;",
-        "import com.example.annotations.MyAnnotation;",
-        "",
-        "public class AnnotationFactory {",
-        "  @AutoAnnotation",
-        "  public static MyAnnotation newMyAnnotation(int[] value) {",
-        "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value);",
-        "  }",
-        "}");
+    JavaFileObject myAnnotationJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.annotations.MyAnnotation",
+            "package com.example.annotations;",
+            "",
+            "import com.google.common.annotations.GwtCompatible;",
+            "",
+            "@GwtCompatible",
+            "public @interface MyAnnotation {",
+            "  int[] value();",
+            "}");
+    JavaFileObject gwtCompatibleJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.google.common.annotations.GwtCompatible",
+            "package com.google.common.annotations;",
+            "",
+            "public @interface GwtCompatible {}");
+    JavaFileObject annotationFactoryJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.factories.AnnotationFactory",
+            "package com.example.factories;",
+            "",
+            "import com.google.auto.value.AutoAnnotation;",
+            "import com.example.annotations.MyAnnotation;",
+            "",
+            "public class AnnotationFactory {",
+            "  @AutoAnnotation",
+            "  public static MyAnnotation newMyAnnotation(int[] value) {",
+            "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value);",
+            "  }",
+            "}");
     JavaFileObject expectedOutput =
         JavaFileObjects.forSourceLines(
             "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation",
             "package com.example.factories;",
             "",
             "import com.example.annotations.MyAnnotation;",
+            "import java.io.Serializable",
             "import java.util.Arrays;",
-            generatedImportRule.importGeneratedAnnotationType(),
+            GeneratedImport.importGeneratedAnnotationType(),
             "",
             "@Generated(\"" + AutoAnnotationProcessor.class.getName() + "\")",
-            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation {",
+            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation,"
+                + " Serializable {",
+            "  private static final long serialVersionUID = -8116050813861599066L;",
             "  private final int[] value;",
             "",
             "  AutoAnnotation_AnnotationFactory_newMyAnnotation(int[] value) {",
@@ -275,66 +280,71 @@ public class AutoAnnotationCompilationTest {
             "    }",
             "    if (o instanceof MyAnnotation) {",
             "      MyAnnotation that = (MyAnnotation) o;",
-            "      return (Arrays.equals(value,",
+            "      return Arrays.equals(value,",
             "          (that instanceof AutoAnnotation_AnnotationFactory_newMyAnnotation)",
             "              ? ((AutoAnnotation_AnnotationFactory_newMyAnnotation) that).value",
-            "              : that.value()));",
+            "              : that.value());",
             "    }",
             "    return false;",
             "  }",
             "",
             "  @Override public int hashCode() {",
             "    return ",
-            "        + (" + 127 * "value".hashCode() + " ^ (Arrays.hashCode(value)));",
+            "        + (" + 127 * "value".hashCode() + " ^ Arrays.hashCode(value));",
             "  }",
             "}");
-    assert_().about(javaSources())
-        .that(ImmutableList.of(
-            annotationFactoryJavaFile, myAnnotationJavaFile, gwtCompatibleJavaFile))
-        .processedWith(new AutoAnnotationProcessor())
-        .compilesWithoutError()
-        .and().generatesSources(expectedOutput);
+    Compilation compilation =
+        javac()
+            .withProcessors(new AutoAnnotationProcessor())
+            .withOptions("-A" + Nullables.NULLABLE_OPTION + "=")
+            .compile(annotationFactoryJavaFile, myAnnotationJavaFile, gwtCompatibleJavaFile);
+    assertThat(compilation).succeededWithoutWarnings();
+    assertThat(compilation)
+        .generatedSourceFile(
+            "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation")
+        .hasSourceEquivalentTo(expectedOutput);
   }
 
   @Test
   public void testCollectionsForArrays() {
-    JavaFileObject myAnnotationJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.annotations.MyAnnotation",
-        "package com.example.annotations;",
-        "",
-        "import com.example.enums.MyEnum;",
-        "",
-        "public @interface MyAnnotation {",
-        "  int[] value();",
-        "  MyEnum[] enums() default {};",
-        "}"
-    );
-    JavaFileObject myEnumJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.enums.MyEnum",
-        "package com.example.enums;",
-        "",
-        "public enum MyEnum {",
-        "  ONE",
-        "}"
-    );
-    JavaFileObject annotationFactoryJavaFile = JavaFileObjects.forSourceLines(
-        "com.example.factories.AnnotationFactory",
-        "package com.example.factories;",
-        "",
-        "import com.google.auto.value.AutoAnnotation;",
-        "import com.example.annotations.MyAnnotation;",
-        "import com.example.enums.MyEnum;",
-        "",
-        "import java.util.List;",
-        "import java.util.Set;",
-        "",
-        "public class AnnotationFactory {",
-        "  @AutoAnnotation",
-        "  public static MyAnnotation newMyAnnotation(",
-        "      List<Integer> value, Set<MyEnum> enums) {",
-        "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value, enums);",
-        "  }",
-        "}");
+    JavaFileObject myAnnotationJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.annotations.MyAnnotation",
+            "package com.example.annotations;",
+            "",
+            "import com.example.enums.MyEnum;",
+            "",
+            "public @interface MyAnnotation {",
+            "  int[] value();",
+            "  MyEnum[] enums() default {};",
+            "}");
+    JavaFileObject myEnumJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.enums.MyEnum",
+            "package com.example.enums;",
+            "",
+            "public enum MyEnum {",
+            "  ONE",
+            "}");
+    JavaFileObject annotationFactoryJavaFile =
+        JavaFileObjects.forSourceLines(
+            "com.example.factories.AnnotationFactory",
+            "package com.example.factories;",
+            "",
+            "import com.google.auto.value.AutoAnnotation;",
+            "import com.example.annotations.MyAnnotation;",
+            "import com.example.enums.MyEnum;",
+            "",
+            "import java.util.List;",
+            "import java.util.Set;",
+            "",
+            "public class AnnotationFactory {",
+            "  @AutoAnnotation",
+            "  public static MyAnnotation newMyAnnotation(",
+            "      List<Integer> value, Set<MyEnum> enums) {",
+            "    return new AutoAnnotation_AnnotationFactory_newMyAnnotation(value, enums);",
+            "  }",
+            "}");
     JavaFileObject expectedOutput =
         JavaFileObjects.forSourceLines(
             "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation",
@@ -342,14 +352,17 @@ public class AutoAnnotationCompilationTest {
             "",
             "import com.example.annotations.MyAnnotation;",
             "import com.example.enums.MyEnum;",
+            "import java.io.Serializable;",
             "import java.util.Arrays;",
             "import java.util.Collection;",
             "import java.util.List;",
             "import java.util.Set;",
-            generatedImportRule.importGeneratedAnnotationType(),
+            GeneratedImport.importGeneratedAnnotationType(),
             "",
             "@Generated(\"" + AutoAnnotationProcessor.class.getName() + "\")",
-            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation {",
+            "final class AutoAnnotation_AnnotationFactory_newMyAnnotation implements MyAnnotation,"
+                + " Serializable {",
+            "  private static final long serialVersionUID = -2102364343628921304L;",
             "  private final int[] value;",
             "  private final MyEnum[] enums;",
             "",
@@ -394,22 +407,22 @@ public class AutoAnnotationCompilationTest {
             "    }",
             "    if (o instanceof MyAnnotation) {",
             "      MyAnnotation that = (MyAnnotation) o;",
-            "      return (Arrays.equals(value,",
+            "      return Arrays.equals(value,",
             "          (that instanceof AutoAnnotation_AnnotationFactory_newMyAnnotation)",
             "              ? ((AutoAnnotation_AnnotationFactory_newMyAnnotation) that).value",
-            "              : that.value()))",
-            "          && (Arrays.equals(enums,",
+            "              : that.value())",
+            "          && Arrays.equals(enums,",
             "          (that instanceof AutoAnnotation_AnnotationFactory_newMyAnnotation)",
             "              ? ((AutoAnnotation_AnnotationFactory_newMyAnnotation) that).enums",
-            "              : that.enums()))",
+            "              : that.enums())",
             "    }",
             "    return false;",
             "  }",
             "",
             "  @Override public int hashCode() {",
             "    return ",
-            "        + (" + 127 * "value".hashCode() + " ^ (Arrays.hashCode(value)))",
-            "        + (" + 127 * "enums".hashCode() + " ^ (Arrays.hashCode(enums)));",
+            "        + (" + 127 * "value".hashCode() + " ^ Arrays.hashCode(value))",
+            "        + (" + 127 * "enums".hashCode() + " ^ Arrays.hashCode(enums));",
             "  }",
             "",
             "  private static int[] intArrayFromCollection(Collection<Integer> c) {",
@@ -421,26 +434,20 @@ public class AutoAnnotationCompilationTest {
             "    return a;",
             "  }",
             "}");
-    assert_().about(javaSources())
-        .that(ImmutableList.of(annotationFactoryJavaFile, myEnumJavaFile, myAnnotationJavaFile))
-        .processedWith(new AutoAnnotationProcessor())
-        .compilesWithoutError()
-        .and().generatesSources(expectedOutput);
+    Compilation compilation =
+        javac()
+            .withProcessors(new AutoAnnotationProcessor())
+            .withOptions("-A" + Nullables.NULLABLE_OPTION + "=")
+            .compile(annotationFactoryJavaFile, myEnumJavaFile, myAnnotationJavaFile);
+    assertThat(compilation).succeededWithoutWarnings();
+    assertThat(compilation)
+        .generatedSourceFile(
+            "com.example.factories.AutoAnnotation_AnnotationFactory_newMyAnnotation")
+        .hasSourceEquivalentTo(expectedOutput);
   }
 
   @Test
-  public void testMissingClass() throws IOException {
-    File tempDir = File.createTempFile("AutoAnnotationCompilationTest", "");
-    assertTrue(tempDir.delete());
-    assertTrue(tempDir.mkdir());
-    try {
-      doTestMissingClass(tempDir);
-    } finally {
-      removeDirectory(tempDir);
-    }
-  }
-
-  private void doTestMissingClass(File tempDir) {
+  public void testMissingClass() {
     // Test that referring to an undefined annotation does not trigger @AutoAnnotation processing.
     // The class Erroneous references an undefined annotation @NotAutoAnnotation. If we didn't have
     // any special treatment of undefined types then we could run into a compiler bug where
@@ -449,45 +456,28 @@ public class AutoAnnotationCompilationTest {
     // being undefined, and we do not get an error complaining that this supposed @AutoAnnotation
     // method is not static. We do need to have @AutoAnnotation appear somewhere so that the
     // processor will run.
-    JavaFileObject erroneousJavaFileObject = JavaFileObjects.forSourceLines(
-        "com.example.annotations.Erroneous",
-        "package com.example.annotations;",
-        "",
-        "import com.google.auto.value.AutoAnnotation;",
-        "",
-        "public class Erroneous {",
-        "  @interface Empty {}",
-        "  @AutoAnnotation static Empty newEmpty() {}",
-        "  @NotAutoAnnotation Empty notNewEmpty() {}",
-        "}"
-    );
-    JavaCompiler javaCompiler = ToolProvider.getSystemJavaCompiler();
-    DiagnosticCollector<JavaFileObject> diagnosticCollector =
-        new DiagnosticCollector<JavaFileObject>();
-    JavaCompiler.CompilationTask compilationTask = javaCompiler.getTask(
-        (Writer) null,
-        (JavaFileManager) null,
-        diagnosticCollector,
-        ImmutableList.of("-d", tempDir.toString()),
-        (Iterable<String>) null,
-        ImmutableList.of(erroneousJavaFileObject));
-    compilationTask.setProcessors(ImmutableList.of(new AutoAnnotationProcessor()));
-    boolean result = compilationTask.call();
-    assertThat(result).isFalse();
-    List<Diagnostic<? extends JavaFileObject>> diagnostics = diagnosticCollector.getDiagnostics();
-    assertThat(diagnostics).isNotEmpty();
-    assertThat(diagnostics.get(0).getMessage(null)).contains("NotAutoAnnotation");
-    assertThat(diagnostics.get(0).getMessage(null)).doesNotContain("static");
-  }
-
-  private static void removeDirectory(File dir) {
-    for (File file : dir.listFiles()) {
-      if (file.isDirectory()) {
-        removeDirectory(file);
-      } else {
-        assertTrue(file.delete());
-      }
-    }
-    assertTrue(dir.delete());
+    JavaFileObject erroneousJavaFileObject =
+        JavaFileObjects.forSourceLines(
+            "com.example.annotations.Erroneous",
+            "package com.example.annotations;",
+            "",
+            "import com.google.auto.value.AutoAnnotation;",
+            "",
+            "public class Erroneous {",
+            "  @interface Empty {}",
+            "  @AutoAnnotation static Empty newEmpty() {}",
+            "  @NotAutoAnnotation Empty notNewEmpty() {}",
+            "}");
+    Compilation compilation =
+        javac().withProcessors(new AutoAnnotationProcessor()).compile(erroneousJavaFileObject);
+    assertThat(compilation)
+        .hadErrorContaining("NotAutoAnnotation")
+        .inFile(erroneousJavaFileObject)
+        .onLineContaining("@NotAutoAnnotation");
+    assertThat(
+            compilation.errors().stream()
+                .map(diag -> diag.getMessage(null))
+                .filter(m -> m.contains("static")))
+        .isEmpty();
   }
 }
